@@ -22,7 +22,8 @@ from bleak.backends.device import BLEDevice
 from bleak.backends.service import BleakGATTServiceCollection
 from bleak.exc import BleakError
 
-from homeassistant.core import CALLBACK_TYPE
+from homeassistant.components.bluetooth import async_scanner_by_source
+from homeassistant.core import CALLBACK_TYPE, HomeAssistant
 
 from ..domain_data import DomainData
 from .characteristic import BleakGATTCharacteristicESPHome
@@ -35,6 +36,13 @@ DISCONNECT_TIMEOUT = 5.0
 CONNECT_FREE_SLOT_TIMEOUT = 2.0
 GATT_READ_TIMEOUT = 30.0
 
+# CCCD (Characteristic Client Config Descriptor)
+CCCD_UUID = "00002902-0000-1000-8000-00805f9b34fb"
+CCCD_NOTIFY_BYTES = b"\x01\x00"
+CCCD_INDICATE_BYTES = b"\x02\x00"
+
+MIN_BLUETOOTH_PROXY_VERSION_HAS_CACHE = 3
+
 DEFAULT_MAX_WRITE_WITHOUT_RESPONSE = DEFAULT_MTU - GATT_HEADER_SIZE
 _LOGGER = logging.getLogger(__name__)
 
@@ -95,15 +103,17 @@ def api_error_as_bleak_error(func: _WrapFuncType) -> _WrapFuncType:
             # Because callbacks are delivered asynchronously it's possible
             # that we find out about the disconnection during the operation
             # before the callback is delivered.
+
+            # pylint: disable=protected-access
             if ex.error.error == -1:
                 _LOGGER.debug(
                     "%s: %s - %s: BLE device disconnected during %s operation",
-                    self._source,  # pylint: disable=protected-access
-                    self._ble_device.name,  # pylint: disable=protected-access
-                    self._ble_device.address,  # pylint: disable=protected-access
+                    self._source,
+                    self._ble_device.name,
+                    self._ble_device.address,
                     func.__name__,
                 )
-                self._async_ble_device_disconnected()  # pylint: disable=protected-access
+                self._async_ble_device_disconnected()
             raise BleakError(str(ex)) from ex
         except APIConnectionError as err:
             raise BleakError(str(err)) from err
@@ -120,19 +130,26 @@ class ESPHomeClient(BaseBleakClient):
         """Initialize the ESPHomeClient."""
         assert isinstance(address_or_ble_device, BLEDevice)
         super().__init__(address_or_ble_device, *args, **kwargs)
+        self._hass: HomeAssistant = kwargs["hass"]
         self._ble_device = address_or_ble_device
         self._address_as_int = mac_to_int(self._ble_device.address)
         assert self._ble_device.details is not None
         self._source = self._ble_device.details["source"]
-        self.domain_data = DomainData.get(kwargs["hass"])
+        self.domain_data = DomainData.get(self._hass)
         config_entry = self.domain_data.get_by_unique_id(self._source)
         self.entry_data = self.domain_data.get_entry_data(config_entry)
         self._client = self.entry_data.client
         self._is_connected = False
         self._mtu: int | None = None
         self._cancel_connection_state: CALLBACK_TYPE | None = None
-        self._notify_cancels: dict[int, Callable[[], Coroutine[Any, Any, None]]] = {}
+        self._notify_cancels: dict[
+            int, tuple[Callable[[], Coroutine[Any, Any, None]], Callable[[], None]]
+        ] = {}
         self._disconnected_event: asyncio.Event | None = None
+        device_info = self.entry_data.device_info
+        assert device_info is not None
+        self._connection_version = device_info.bluetooth_proxy_version
+        self._address_type = address_or_ble_device.details["address_type"]
 
     def __str__(self) -> str:
         """Return the string representation of the client."""
@@ -154,15 +171,22 @@ class ESPHomeClient(BaseBleakClient):
             )
         self._cancel_connection_state = None
 
-    def _async_ble_device_disconnected(self) -> None:
-        """Handle the BLE device disconnecting from the ESP."""
-        was_connected = self._is_connected
+    def _async_disconnected_cleanup(self) -> None:
+        """Clean up on disconnect."""
         self.services = BleakGATTServiceCollection()  # type: ignore[no-untyped-call]
         self._is_connected = False
+        for _, notify_abort in self._notify_cancels.values():
+            notify_abort()
         self._notify_cancels.clear()
         if self._disconnected_event:
             self._disconnected_event.set()
             self._disconnected_event = None
+        self._unsubscribe_connection_state()
+
+    def _async_ble_device_disconnected(self) -> None:
+        """Handle the BLE device disconnecting from the ESP."""
+        was_connected = self._is_connected
+        self._async_disconnected_cleanup()
         if was_connected:
             _LOGGER.debug(
                 "%s: %s - %s: BLE device disconnected",
@@ -171,7 +195,6 @@ class ESPHomeClient(BaseBleakClient):
                 self._ble_device.address,
             )
             self._async_call_bleak_disconnected_callback()
-        self._unsubscribe_connection_state()
 
     def _async_esp_disconnected(self) -> None:
         """Handle the esp32 client disconnecting from hass."""
@@ -202,7 +225,14 @@ class ESPHomeClient(BaseBleakClient):
             Boolean representing connection status.
         """
         await self._wait_for_free_connection_slot(CONNECT_FREE_SLOT_TIMEOUT)
-
+        entry_data = self.entry_data
+        self._mtu = entry_data.get_gatt_mtu_cache(self._address_as_int)
+        has_cache = bool(
+            dangerous_use_bleak_cache
+            and self._connection_version >= MIN_BLUETOOTH_PROXY_VERSION_HAS_CACHE
+            and entry_data.get_gatt_services_cache(self._address_as_int)
+            and self._mtu
+        )
         connected_future: asyncio.Future[bool] = asyncio.Future()
 
         def _on_bluetooth_connection_state(
@@ -220,7 +250,9 @@ class ESPHomeClient(BaseBleakClient):
             )
             if connected:
                 self._is_connected = True
-                self._mtu = mtu
+                if not self._mtu:
+                    self._mtu = mtu
+                    entry_data.set_gatt_mtu_cache(self._address_as_int, mtu)
             else:
                 self._async_ble_device_disconnected()
 
@@ -254,24 +286,58 @@ class ESPHomeClient(BaseBleakClient):
                 self._ble_device.name,
                 self._ble_device.address,
             )
-            self.entry_data.disconnect_callbacks.append(self._async_esp_disconnected)
+            entry_data.disconnect_callbacks.append(self._async_esp_disconnected)
             connected_future.set_result(connected)
 
         timeout = kwargs.get("timeout", self._timeout)
-        self._cancel_connection_state = await self._client.bluetooth_device_connect(
-            self._address_as_int,
-            _on_bluetooth_connection_state,
-            timeout=timeout,
-        )
-        await connected_future
-        await self.get_services(dangerous_use_bleak_cache=dangerous_use_bleak_cache)
+        if not (scanner := async_scanner_by_source(self._hass, self._source)):
+            raise BleakError("Scanner disappeared for {self._source}")
+        with scanner.connecting():
+            try:
+                self._cancel_connection_state = (
+                    await self._client.bluetooth_device_connect(
+                        self._address_as_int,
+                        _on_bluetooth_connection_state,
+                        timeout=timeout,
+                        has_cache=has_cache,
+                        version=self._connection_version,
+                        address_type=self._address_type,
+                    )
+                )
+            except Exception:
+                with contextlib.suppress(BleakError):
+                    # If the connect call throws an exception,
+                    # we need to make sure we await the future
+                    # to avoid a warning about an un-retrieved
+                    # exception since we prefer to raise the
+                    # exception from the connect call as it
+                    # will be more descriptive.
+                    if connected_future.done():
+                        await connected_future
+                connected_future.cancel()
+                raise
+            await connected_future
+
+        try:
+            await self.get_services(dangerous_use_bleak_cache=dangerous_use_bleak_cache)
+        except asyncio.CancelledError:
+            # On cancel we must still raise cancelled error
+            # to avoid blocking the cancellation even if the
+            # disconnect call fails.
+            with contextlib.suppress(Exception):
+                await self.disconnect()
+            raise
+        except Exception:
+            await self.disconnect()
+            raise
+
         self._disconnected_event = asyncio.Event()
         return True
 
     @api_error_as_bleak_error
     async def disconnect(self) -> bool:
         """Disconnect from the peripheral device."""
-        self._unsubscribe_connection_state()
+        self._async_disconnected_cleanup()
         await self._client.bluetooth_device_disconnect(self._address_as_int)
         await self._wait_for_free_connection_slot(DISCONNECT_TIMEOUT)
         return True
@@ -322,9 +388,13 @@ class ESPHomeClient(BaseBleakClient):
         """
         address_as_int = self._address_as_int
         entry_data = self.entry_data
-        if dangerous_use_bleak_cache and (
-            cached_services := entry_data.get_gatt_services_cache(address_as_int)
-        ):
+        # If the connection version >= 3, we must use the cache
+        # because the esp has already wiped the services list to
+        # save memory.
+        if (
+            self._connection_version >= MIN_BLUETOOTH_PROXY_VERSION_HAS_CACHE
+            or dangerous_use_bleak_cache
+        ) and (cached_services := entry_data.get_gatt_services_cache(address_as_int)):
             _LOGGER.debug(
                 "%s: %s - %s: Cached services hit",
                 self._source,
@@ -370,6 +440,12 @@ class ESPHomeClient(BaseBleakClient):
                             characteristic.handle,
                         )
                     )
+
+        if not esphome_services.services:
+            # If we got no services, we must have disconnected
+            # or something went wrong on the ESP32's BLE stack.
+            raise BleakError("Failed to get services from remote esp")
+
         self.services = services
         _LOGGER.debug(
             "%s: %s - %s: Cached services saved",
@@ -392,6 +468,11 @@ class ESPHomeClient(BaseBleakClient):
             raise BleakError(f"Characteristic {char_specifier} was not found!")
         return characteristic
 
+    async def clear_cache(self) -> None:
+        """Clear the GATT cache."""
+        self.entry_data.clear_gatt_services_cache(self._address_as_int)
+        self.entry_data.clear_gatt_mtu_cache(self._address_as_int)
+
     @verify_connected
     @api_error_as_bleak_error
     async def read_gatt_char(
@@ -494,12 +575,52 @@ class ESPHomeClient(BaseBleakClient):
                 f"characteristic:{characteristic.uuid} "
                 f"handle:{ble_handle}"
             )
-        cancel_coro = await self._client.bluetooth_gatt_start_notify(
+        if (
+            "notify" not in characteristic.properties
+            and "indicate" not in characteristic.properties
+        ):
+            raise BleakError(
+                f"Characteristic {characteristic.uuid} does not have notify or indicate property set."
+            )
+
+        self._notify_cancels[
+            ble_handle
+        ] = await self._client.bluetooth_gatt_start_notify(
             self._address_as_int,
             ble_handle,
             lambda handle, data: callback(data),
         )
-        self._notify_cancels[ble_handle] = cancel_coro
+
+        if self._connection_version < MIN_BLUETOOTH_PROXY_VERSION_HAS_CACHE:
+            return
+
+        # For connection v3 we are responsible for enabling notifications
+        # on the cccd (characteristic client config descriptor) handle since
+        # the esp32 will not have resolved the characteristic descriptors to
+        # save memory since doing so can exhaust the memory and cause a soft
+        # reset
+        cccd_descriptor = characteristic.get_descriptor(CCCD_UUID)
+        if not cccd_descriptor:
+            raise BleakError(
+                f"Characteristic {characteristic.uuid} does not have a "
+                "characteristic client config descriptor."
+            )
+
+        _LOGGER.debug(
+            "%s: %s - %s: Writing to CCD descriptor %s for notifications with properties=%s",
+            self._source,
+            self._ble_device.name,
+            self._ble_device.address,
+            cccd_descriptor.handle,
+            characteristic.properties,
+        )
+        supports_notify = "notify" in characteristic.properties
+        await self._client.bluetooth_gatt_write_descriptor(
+            self._address_as_int,
+            cccd_descriptor.handle,
+            CCCD_NOTIFY_BYTES if supports_notify else CCCD_INDICATE_BYTES,
+            wait_for_response=False,
+        )
 
     @api_error_as_bleak_error
     async def stop_notify(
@@ -516,5 +637,18 @@ class ESPHomeClient(BaseBleakClient):
         characteristic = self._resolve_characteristic(char_specifier)
         # Do not raise KeyError if notifications are not enabled on this characteristic
         # to be consistent with the behavior of the BlueZ backend
-        if coro := self._notify_cancels.pop(characteristic.handle, None):
-            await coro()
+        if notify_cancel := self._notify_cancels.pop(characteristic.handle, None):
+            notify_stop, _ = notify_cancel
+            await notify_stop()
+
+    def __del__(self) -> None:
+        """Destructor to make sure the connection state is unsubscribed."""
+        if self._cancel_connection_state:
+            _LOGGER.warning(
+                "%s: %s - %s: ESPHomeClient bleak client was not properly disconnected before destruction",
+                self._source,
+                self._ble_device.name,
+                self._ble_device.address,
+            )
+        if not self._hass.loop.is_closed():
+            self._hass.loop.call_soon_threadsafe(self._async_disconnected_cleanup)
