@@ -1,7 +1,7 @@
 """Support for AVM FRITZ!SmartHome lightbulbs."""
 from __future__ import annotations
 
-from typing import Any
+from typing import Any, cast
 
 from requests.exceptions import HTTPError
 
@@ -16,7 +16,7 @@ from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
-from . import FritzBoxEntity
+from . import FritzboxDataUpdateCoordinator, FritzBoxDeviceEntity
 from .const import (
     COLOR_MODE,
     COLOR_TEMP_MODE,
@@ -24,7 +24,6 @@ from .const import (
     DOMAIN as FRITZBOX_DOMAIN,
     LOGGER,
 )
-from .coordinator import FritzboxDataUpdateCoordinator
 
 SUPPORTED_COLOR_MODES = {ColorMode.COLOR_TEMP, ColorMode.HS}
 
@@ -34,9 +33,11 @@ async def async_setup_entry(
 ) -> None:
     """Set up the FRITZ!SmartHome light from ConfigEntry."""
     entities: list[FritzboxLight] = []
-    coordinator = hass.data[FRITZBOX_DOMAIN][entry.entry_id][CONF_COORDINATOR]
+    coordinator: FritzboxDataUpdateCoordinator = hass.data[FRITZBOX_DOMAIN][
+        entry.entry_id
+    ][CONF_COORDINATOR]
 
-    for ain, device in coordinator.data.items():
+    for ain, device in coordinator.data.devices.items():
         if not device.has_lightbulb:
             continue
 
@@ -58,7 +59,7 @@ async def async_setup_entry(
     async_add_entities(entities)
 
 
-class FritzboxLight(FritzBoxEntity, LightEntity):
+class FritzboxLight(FritzBoxDeviceEntity, LightEntity):
     """The light class for FRITZ!SmartHome lightbulbs."""
 
     def __init__(
@@ -76,7 +77,7 @@ class FritzboxLight(FritzBoxEntity, LightEntity):
 
         # Fritz!DECT 500 only supports 12 values for hue, with 3 saturations each.
         # Map supported colors to dict {hue: [sat1, sat2, sat3]} for easier lookup
-        self._supported_hs = {}
+        self._supported_hs: dict[int, list[int]] = {}
         for values in supported_colors.values():
             hue = int(values[0][0])
             self._supported_hs[hue] = [
@@ -88,36 +89,36 @@ class FritzboxLight(FritzBoxEntity, LightEntity):
     @property
     def is_on(self) -> bool:
         """If the light is currently on or off."""
-        return self.device.state  # type: ignore [no-any-return]
+        return self.data.state  # type: ignore [no-any-return]
 
     @property
     def brightness(self) -> int:
         """Return the current Brightness."""
-        return self.device.level  # type: ignore [no-any-return]
+        return self.data.level  # type: ignore [no-any-return]
 
     @property
     def hs_color(self) -> tuple[float, float] | None:
         """Return the hs color value."""
-        if self.device.color_mode != COLOR_MODE:
+        if self.data.color_mode != COLOR_MODE:
             return None
 
-        hue = self.device.hue
-        saturation = self.device.saturation
+        hue = self.data.hue
+        saturation = self.data.saturation
 
         return (hue, float(saturation) * 100.0 / 255.0)
 
     @property
     def color_temp_kelvin(self) -> int | None:
         """Return the CT color value."""
-        if self.device.color_mode != COLOR_TEMP_MODE:
+        if self.data.color_mode != COLOR_TEMP_MODE:
             return None
 
-        return self.device.color_temp  # type: ignore [no-any-return]
+        return self.data.color_temp  # type: ignore [no-any-return]
 
     @property
     def color_mode(self) -> ColorMode:
         """Return the color mode of the light."""
-        if self.device.color_mode == COLOR_MODE:
+        if self.data.color_mode == COLOR_MODE:
             return ColorMode.HS
         return ColorMode.COLOR_TEMP
 
@@ -130,16 +131,18 @@ class FritzboxLight(FritzBoxEntity, LightEntity):
         """Turn the light on."""
         if kwargs.get(ATTR_BRIGHTNESS) is not None:
             level = kwargs[ATTR_BRIGHTNESS]
-            await self.hass.async_add_executor_job(self.device.set_level, level)
+            await self.hass.async_add_executor_job(self.data.set_level, level)
         if kwargs.get(ATTR_HS_COLOR) is not None:
             # Try setunmappedcolor first. This allows free color selection,
             # but we don't know if its supported by all devices.
             try:
                 # HA gives 0..360 for hue, fritz light only supports 0..359
                 unmapped_hue = int(kwargs[ATTR_HS_COLOR][0] % 360)
-                unmapped_saturation = round(kwargs[ATTR_HS_COLOR][1] * 255.0 / 100.0)
+                unmapped_saturation = round(
+                    cast(float, kwargs[ATTR_HS_COLOR][1]) * 255.0 / 100.0
+                )
                 await self.hass.async_add_executor_job(
-                    self.device.set_unmapped_color, (unmapped_hue, unmapped_saturation)
+                    self.data.set_unmapped_color, (unmapped_hue, unmapped_saturation)
                 )
             # This will raise 400 BAD REQUEST if the setunmappedcolor is not available
             except HTTPError as err:
@@ -157,18 +160,18 @@ class FritzboxLight(FritzBoxEntity, LightEntity):
                     key=lambda x: abs(x - unmapped_saturation),
                 )
                 await self.hass.async_add_executor_job(
-                    self.device.set_color, (hue, saturation)
+                    self.data.set_color, (hue, saturation)
                 )
 
         if kwargs.get(ATTR_COLOR_TEMP_KELVIN) is not None:
             await self.hass.async_add_executor_job(
-                self.device.set_color_temp, kwargs[ATTR_COLOR_TEMP_KELVIN]
+                self.data.set_color_temp, kwargs[ATTR_COLOR_TEMP_KELVIN]
             )
 
-        await self.hass.async_add_executor_job(self.device.set_state_on)
+        await self.hass.async_add_executor_job(self.data.set_state_on)
         await self.coordinator.async_refresh()
 
     async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn the light off."""
-        await self.hass.async_add_executor_job(self.device.set_state_off)
+        await self.hass.async_add_executor_job(self.data.set_state_off)
         await self.coordinator.async_refresh()
