@@ -1,6 +1,8 @@
 """Constants for the Frontier Silicon Media Player integration."""
-
 DOMAIN = "frontier_silicon"
 
 DEFAULT_PIN = "1234"
 DEFAULT_PORT = 80
+
+MEDIA_CONTENT_ID_PRESET = "preset"
+MEDIA_CONTENT_ID_CHANNELS = "channels"
