@@ -1,5 +1,9 @@
 """Support for MQTT lights."""
+from __future__ import annotations
+
+from collections.abc import Callable
 import logging
+from typing import Any, cast
 
 import voluptuous as vol
 
@@ -24,6 +28,7 @@ from homeassistant.components.light import (
     LightEntityFeature,
     valid_supported_color_modes,
 )
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     CONF_NAME,
     CONF_OPTIMISTIC,
@@ -32,9 +37,11 @@ from homeassistant.const import (
     CONF_VALUE_TEMPLATE,
     STATE_ON,
 )
-from homeassistant.core import callback
+from homeassistant.core import HomeAssistant, callback
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.restore_state import RestoreEntity
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 import homeassistant.util.color as color_util
 
 from .. import subscription
@@ -50,8 +57,17 @@ from ..const import (
 )
 from ..debug_info import log_messages
 from ..mixins import MQTT_ENTITY_COMMON_SCHEMA, MqttEntity
-from ..models import MqttCommandTemplate, MqttValueTemplate
-from ..util import valid_publish_topic, valid_subscribe_topic
+from ..models import (
+    MessageCallbackType,
+    MqttCommandTemplate,
+    MqttValueTemplate,
+    PayloadSentinel,
+    PublishPayloadType,
+    ReceiveMessage,
+    ReceivePayloadType,
+    TemplateVarsType,
+)
+from ..util import get_mqtt_data, valid_publish_topic, valid_subscribe_topic
 from .schema import MQTT_LIGHT_SCHEMA_SCHEMA
 
 _LOGGER = logging.getLogger(__name__)
@@ -208,7 +224,7 @@ _PLATFORM_SCHEMA_BASE = (
     .extend(MQTT_LIGHT_SCHEMA_SCHEMA.schema)
 )
 
-# The use of PLATFORM_SCHEMA is deprecated in HA Core 2022.6
+# The use of PLATFORM_SCHEMA was deprecated in HA Core 2022.6
 PLATFORM_SCHEMA_BASIC = vol.All(
     cv.PLATFORM_SCHEMA.extend(_PLATFORM_SCHEMA_BASE.schema),
 )
@@ -237,8 +253,12 @@ PLATFORM_SCHEMA_MODERN_BASIC = vol.All(
 
 
 async def async_setup_entity_basic(
-    hass, config, async_add_entities, config_entry, discovery_data=None
-):
+    hass: HomeAssistant,
+    config: ConfigType,
+    async_add_entities: AddEntitiesCallback,
+    config_entry: ConfigEntry,
+    discovery_data: DiscoveryInfoType | None,
+) -> None:
     """Set up a MQTT Light."""
     async_add_entities([MqttLight(hass, config, config_entry, discovery_data)])
 
@@ -248,49 +268,50 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
 
     _entity_id_format = ENTITY_ID_FORMAT
     _attributes_extra_blocked = MQTT_LIGHT_ATTRIBUTES_BLOCKED
+    _topic: dict[str, str | None]
+    _payload: dict[str, str]
+    _command_templates: dict[
+        str, Callable[[PublishPayloadType, TemplateVarsType], PublishPayloadType]
+    ]
+    _value_templates: dict[
+        str, Callable[[ReceivePayloadType, ReceivePayloadType], ReceivePayloadType]
+    ]
+    _optimistic: bool
+    _optimistic_brightness: bool
+    _optimistic_color_mode: bool
+    _optimistic_color_temp: bool
+    _optimistic_effect: bool
+    _optimistic_hs_color: bool
+    _optimistic_rgb_color: bool
+    _optimistic_rgbw_color: bool
+    _optimistic_rgbww_color: bool
+    _optimistic_xy_color: bool
 
-    def __init__(self, hass, config, config_entry, discovery_data):
+    def __init__(
+        self,
+        hass: HomeAssistant,
+        config: ConfigType,
+        config_entry: ConfigEntry,
+        discovery_data: DiscoveryInfoType | None,
+    ) -> None:
         """Initialize MQTT light."""
-        self._brightness = None
-        self._color_mode = None
-        self._color_temp = None
-        self._effect = None
-        self._hs_color = None
-        self._rgb_color = None
-        self._rgbw_color = None
-        self._rgbww_color = None
-        self._state = None
-        self._supported_color_modes = None
-        self._xy_color = None
-
-        self._topic = None
-        self._payload = None
-        self._command_templates = None
-        self._value_templates = None
-        self._optimistic = False
-        self._optimistic_brightness = False
-        self._optimistic_color_mode = False
-        self._optimistic_color_temp = False
-        self._optimistic_effect = False
-        self._optimistic_hs_color = False
-        self._optimistic_rgb_color = False
-        self._optimistic_rgbw_color = False
-        self._optimistic_rgbww_color = False
-        self._optimistic_xy_color = False
-
         MqttEntity.__init__(self, hass, config, config_entry, discovery_data)
 
     @staticmethod
-    def config_schema():
+    def config_schema() -> vol.Schema:
         """Return the config schema."""
         return DISCOVERY_SCHEMA_BASIC
 
-    def _setup_from_config(self, config):
+    def _setup_from_config(self, config: ConfigType) -> None:
         """(Re)Setup the entity."""
+        self._attr_min_mireds = config.get(CONF_MIN_MIREDS, super().min_mireds)
+        self._attr_max_mireds = config.get(CONF_MAX_MIREDS, super().max_mireds)
+        self._attr_effect_list = config.get(CONF_EFFECT_LIST)
+
         if CONF_STATE_VALUE_TEMPLATE not in config and CONF_VALUE_TEMPLATE in config:
             config[CONF_STATE_VALUE_TEMPLATE] = config[CONF_VALUE_TEMPLATE]
 
-        topic = {
+        topic: dict[str, str | None] = {
             key: config.get(key)
             for key in (
                 CONF_BRIGHTNESS_COMMAND_TOPIC,
@@ -318,32 +339,19 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
         self._topic = topic
         self._payload = {"on": config[CONF_PAYLOAD_ON], "off": config[CONF_PAYLOAD_OFF]}
 
-        value_templates = {}
-        for key in VALUE_TEMPLATE_KEYS:
-            value_templates[key] = None
-        if CONF_VALUE_TEMPLATE in config:
-            value_templates = {
-                key: config.get(CONF_VALUE_TEMPLATE) for key in VALUE_TEMPLATE_KEYS
-            }
-        for key in VALUE_TEMPLATE_KEYS & config.keys():
-            value_templates[key] = config[key]
         self._value_templates = {
             key: MqttValueTemplate(
-                template, entity=self
+                config.get(key), entity=self
             ).async_render_with_possible_json_value
-            for key, template in value_templates.items()
+            for key in VALUE_TEMPLATE_KEYS
         }
 
-        command_templates = {}
-        for key in COMMAND_TEMPLATE_KEYS:
-            command_templates[key] = None
-        for key in COMMAND_TEMPLATE_KEYS & config.keys():
-            command_templates[key] = MqttCommandTemplate(
-                config[key], entity=self
-            ).async_render
-        self._command_templates = command_templates
+        self._command_templates = {
+            key: MqttCommandTemplate(config.get(key), entity=self).async_render
+            for key in COMMAND_TEMPLATE_KEYS
+        }
 
-        optimistic = config[CONF_OPTIMISTIC]
+        optimistic: bool = config[CONF_OPTIMISTIC]
         self._optimistic_color_mode = (
             optimistic or topic[CONF_COLOR_MODE_STATE_TOPIC] is None
         )
@@ -370,50 +378,57 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
         self._optimistic_effect = optimistic or topic[CONF_EFFECT_STATE_TOPIC] is None
         self._optimistic_hs_color = optimistic or topic[CONF_HS_STATE_TOPIC] is None
         self._optimistic_xy_color = optimistic or topic[CONF_XY_STATE_TOPIC] is None
-        supported_color_modes = set()
+        supported_color_modes: set[ColorMode] = set()
         if topic[CONF_COLOR_TEMP_COMMAND_TOPIC] is not None:
             supported_color_modes.add(ColorMode.COLOR_TEMP)
-            self._color_mode = ColorMode.COLOR_TEMP
+            self._attr_color_mode = ColorMode.COLOR_TEMP
         if topic[CONF_HS_COMMAND_TOPIC] is not None:
             supported_color_modes.add(ColorMode.HS)
-            self._color_mode = ColorMode.HS
+            self._attr_color_mode = ColorMode.HS
         if topic[CONF_RGB_COMMAND_TOPIC] is not None:
             supported_color_modes.add(ColorMode.RGB)
-            self._color_mode = ColorMode.RGB
+            self._attr_color_mode = ColorMode.RGB
         if topic[CONF_RGBW_COMMAND_TOPIC] is not None:
             supported_color_modes.add(ColorMode.RGBW)
-            self._color_mode = ColorMode.RGBW
+            self._attr_color_mode = ColorMode.RGBW
         if topic[CONF_RGBWW_COMMAND_TOPIC] is not None:
             supported_color_modes.add(ColorMode.RGBWW)
-            self._color_mode = ColorMode.RGBWW
+            self._attr_color_mode = ColorMode.RGBWW
         if topic[CONF_WHITE_COMMAND_TOPIC] is not None:
             supported_color_modes.add(ColorMode.WHITE)
         if topic[CONF_XY_COMMAND_TOPIC] is not None:
             supported_color_modes.add(ColorMode.XY)
-            self._color_mode = ColorMode.XY
+            self._attr_color_mode = ColorMode.XY
         if len(supported_color_modes) > 1:
-            self._color_mode = ColorMode.UNKNOWN
+            self._attr_color_mode = ColorMode.UNKNOWN
 
         if not supported_color_modes:
             if topic[CONF_BRIGHTNESS_COMMAND_TOPIC] is not None:
-                self._color_mode = ColorMode.BRIGHTNESS
+                self._attr_color_mode = ColorMode.BRIGHTNESS
                 supported_color_modes.add(ColorMode.BRIGHTNESS)
             else:
-                self._color_mode = ColorMode.ONOFF
+                self._attr_color_mode = ColorMode.ONOFF
                 supported_color_modes.add(ColorMode.ONOFF)
 
         # Validate the color_modes configuration
-        self._supported_color_modes = valid_supported_color_modes(supported_color_modes)
+        self._attr_supported_color_modes = valid_supported_color_modes(
+            supported_color_modes
+        )
 
-    def _is_optimistic(self, attribute):
+        self._attr_supported_features = LightEntityFeature(0)
+        if topic[CONF_EFFECT_COMMAND_TOPIC] is not None:
+            self._attr_supported_features |= LightEntityFeature.EFFECT
+
+    def _is_optimistic(self, attribute: str) -> bool:
         """Return True if the attribute is optimistically updated."""
-        return getattr(self, f"_optimistic_{attribute}")
+        attr: bool = getattr(self, f"_optimistic_{attribute}")
+        return attr
 
-    def _prepare_subscribe_topics(self):  # noqa: C901
+    def _prepare_subscribe_topics(self) -> None:  # noqa: C901
         """(Re)Subscribe to topics."""
-        topics = {}
+        topics: dict[str, dict[str, Any]] = {}
 
-        def add_topic(topic, msg_callback):
+        def add_topic(topic: str, msg_callback: MessageCallbackType) -> None:
             """Add a topic."""
             if self._topic[topic] is not None:
                 topics[topic] = {
@@ -425,20 +440,22 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def state_received(msg):
+        def state_received(msg: ReceiveMessage) -> None:
             """Handle new MQTT messages."""
-            payload = self._value_templates[CONF_STATE_VALUE_TEMPLATE](msg.payload)
+            payload = self._value_templates[CONF_STATE_VALUE_TEMPLATE](
+                msg.payload, PayloadSentinel.NONE
+            )
             if not payload:
                 _LOGGER.debug("Ignoring empty state message from '%s'", msg.topic)
                 return
 
             if payload == self._payload["on"]:
-                self._state = True
+                self._attr_is_on = True
             elif payload == self._payload["off"]:
-                self._state = False
+                self._attr_is_on = False
             elif payload == PAYLOAD_NONE:
-                self._state = None
-            self.async_write_ha_state()
+                self._attr_is_on = None
+            get_mqtt_data(self.hass).state_write_requests.write_state_request(self)
 
         if self._topic[CONF_STATE_TOPIC] is not None:
             topics[CONF_STATE_TOPIC] = {
@@ -450,56 +467,64 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def brightness_received(msg):
+        def brightness_received(msg: ReceiveMessage) -> None:
             """Handle new MQTT messages for the brightness."""
             payload = self._value_templates[CONF_BRIGHTNESS_VALUE_TEMPLATE](
-                msg.payload, None
+                msg.payload, PayloadSentinel.DEFAULT
             )
-            if not payload:
+            if payload is PayloadSentinel.DEFAULT or not payload:
                 _LOGGER.debug("Ignoring empty brightness message from '%s'", msg.topic)
                 return
 
             device_value = float(payload)
             percent_bright = device_value / self._config[CONF_BRIGHTNESS_SCALE]
-            self._brightness = percent_bright * 255
-            self.async_write_ha_state()
+            self._attr_brightness = min(round(percent_bright * 255), 255)
+
+            get_mqtt_data(self.hass).state_write_requests.write_state_request(self)
 
         add_topic(CONF_BRIGHTNESS_STATE_TOPIC, brightness_received)
 
-        def _rgbx_received(msg, template, color_mode, convert_color):
+        def _rgbx_received(
+            msg: ReceiveMessage,
+            template: str,
+            color_mode: ColorMode,
+            convert_color: Callable[..., tuple[int, ...]],
+        ) -> tuple[int, ...] | None:
             """Handle new MQTT messages for RGBW and RGBWW."""
-            payload = self._value_templates[template](msg.payload, None)
-            if not payload:
+            payload = self._value_templates[template](
+                msg.payload, PayloadSentinel.DEFAULT
+            )
+            if payload is PayloadSentinel.DEFAULT or not payload:
                 _LOGGER.debug(
                     "Ignoring empty %s message from '%s'", color_mode, msg.topic
                 )
                 return None
-            color = tuple(int(val) for val in payload.split(","))
+            color = tuple(int(val) for val in str(payload).split(","))
             if self._optimistic_color_mode:
-                self._color_mode = color_mode
+                self._attr_color_mode = color_mode
             if self._topic[CONF_BRIGHTNESS_STATE_TOPIC] is None:
                 rgb = convert_color(*color)
                 percent_bright = float(color_util.color_RGB_to_hsv(*rgb)[2]) / 100.0
-                self._brightness = percent_bright * 255
+                self._attr_brightness = min(round(percent_bright * 255), 255)
             return color
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def rgb_received(msg):
+        def rgb_received(msg: ReceiveMessage) -> None:
             """Handle new MQTT messages for RGB."""
             rgb = _rgbx_received(
                 msg, CONF_RGB_VALUE_TEMPLATE, ColorMode.RGB, lambda *x: x
             )
-            if not rgb:
+            if rgb is None:
                 return
-            self._rgb_color = rgb
-            self.async_write_ha_state()
+            self._attr_rgb_color = cast(tuple[int, int, int], rgb)
+            get_mqtt_data(self.hass).state_write_requests.write_state_request(self)
 
         add_topic(CONF_RGB_STATE_TOPIC, rgb_received)
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def rgbw_received(msg):
+        def rgbw_received(msg: ReceiveMessage) -> None:
             """Handle new MQTT messages for RGBW."""
             rgbw = _rgbx_received(
                 msg,
@@ -507,16 +532,16 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
                 ColorMode.RGBW,
                 color_util.color_rgbw_to_rgb,
             )
-            if not rgbw:
+            if rgbw is None:
                 return
-            self._rgbw_color = rgbw
-            self.async_write_ha_state()
+            self._attr_rgbw_color = cast(tuple[int, int, int, int], rgbw)
+            get_mqtt_data(self.hass).state_write_requests.write_state_request(self)
 
         add_topic(CONF_RGBW_STATE_TOPIC, rgbw_received)
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def rgbww_received(msg):
+        def rgbww_received(msg: ReceiveMessage) -> None:
             """Handle new MQTT messages for RGBWW."""
             rgbww = _rgbx_received(
                 msg,
@@ -524,77 +549,79 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
                 ColorMode.RGBWW,
                 color_util.color_rgbww_to_rgb,
             )
-            if not rgbww:
+            if rgbww is None:
                 return
-            self._rgbww_color = rgbww
-            self.async_write_ha_state()
+            self._attr_rgbww_color = cast(tuple[int, int, int, int, int], rgbww)
+            get_mqtt_data(self.hass).state_write_requests.write_state_request(self)
 
         add_topic(CONF_RGBWW_STATE_TOPIC, rgbww_received)
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def color_mode_received(msg):
+        def color_mode_received(msg: ReceiveMessage) -> None:
             """Handle new MQTT messages for color mode."""
             payload = self._value_templates[CONF_COLOR_MODE_VALUE_TEMPLATE](
-                msg.payload, None
+                msg.payload, PayloadSentinel.DEFAULT
             )
-            if not payload:
+            if payload is PayloadSentinel.DEFAULT or not payload:
                 _LOGGER.debug("Ignoring empty color mode message from '%s'", msg.topic)
                 return
 
-            self._color_mode = payload
-            self.async_write_ha_state()
+            self._attr_color_mode = str(payload)
+            get_mqtt_data(self.hass).state_write_requests.write_state_request(self)
 
         add_topic(CONF_COLOR_MODE_STATE_TOPIC, color_mode_received)
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def color_temp_received(msg):
+        def color_temp_received(msg: ReceiveMessage) -> None:
             """Handle new MQTT messages for color temperature."""
             payload = self._value_templates[CONF_COLOR_TEMP_VALUE_TEMPLATE](
-                msg.payload, None
+                msg.payload, PayloadSentinel.DEFAULT
             )
-            if not payload:
+            if payload is PayloadSentinel.DEFAULT or not payload:
                 _LOGGER.debug("Ignoring empty color temp message from '%s'", msg.topic)
                 return
 
             if self._optimistic_color_mode:
-                self._color_mode = ColorMode.COLOR_TEMP
-            self._color_temp = int(payload)
-            self.async_write_ha_state()
+                self._attr_color_mode = ColorMode.COLOR_TEMP
+            self._attr_color_temp = int(payload)
+            get_mqtt_data(self.hass).state_write_requests.write_state_request(self)
 
         add_topic(CONF_COLOR_TEMP_STATE_TOPIC, color_temp_received)
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def effect_received(msg):
+        def effect_received(msg: ReceiveMessage) -> None:
             """Handle new MQTT messages for effect."""
             payload = self._value_templates[CONF_EFFECT_VALUE_TEMPLATE](
-                msg.payload, None
+                msg.payload, PayloadSentinel.DEFAULT
             )
-            if not payload:
+            if payload is PayloadSentinel.DEFAULT or not payload:
                 _LOGGER.debug("Ignoring empty effect message from '%s'", msg.topic)
                 return
 
-            self._effect = payload
-            self.async_write_ha_state()
+            self._attr_effect = str(payload)
+            get_mqtt_data(self.hass).state_write_requests.write_state_request(self)
 
         add_topic(CONF_EFFECT_STATE_TOPIC, effect_received)
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def hs_received(msg):
+        def hs_received(msg: ReceiveMessage) -> None:
             """Handle new MQTT messages for hs color."""
-            payload = self._value_templates[CONF_HS_VALUE_TEMPLATE](msg.payload, None)
-            if not payload:
+            payload = self._value_templates[CONF_HS_VALUE_TEMPLATE](
+                msg.payload, PayloadSentinel.DEFAULT
+            )
+            if payload is PayloadSentinel.DEFAULT or not payload:
                 _LOGGER.debug("Ignoring empty hs message from '%s'", msg.topic)
                 return
             try:
-                hs_color = tuple(float(val) for val in payload.split(",", 2))
+                hs_color = tuple(float(val) for val in str(payload).split(",", 2))
                 if self._optimistic_color_mode:
-                    self._color_mode = ColorMode.HS
-                self._hs_color = hs_color
-                self.async_write_ha_state()
+                    self._attr_color_mode = ColorMode.HS
+                self._attr_hs_color = cast(tuple[float, float], hs_color)
+                get_mqtt_data(self.hass).state_write_requests.write_state_request(self)
             except ValueError:
                 _LOGGER.debug("Failed to parse hs state update: '%s'", payload)
 
@@ -602,18 +629,20 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def xy_received(msg):
+        def xy_received(msg: ReceiveMessage) -> None:
             """Handle new MQTT messages for xy color."""
-            payload = self._value_templates[CONF_XY_VALUE_TEMPLATE](msg.payload, None)
-            if not payload:
+            payload = self._value_templates[CONF_XY_VALUE_TEMPLATE](
+                msg.payload, PayloadSentinel.DEFAULT
+            )
+            if payload is PayloadSentinel.DEFAULT or not payload:
                 _LOGGER.debug("Ignoring empty xy-color message from '%s'", msg.topic)
                 return
 
-            xy_color = tuple(float(val) for val in payload.split(","))
+            xy_color = tuple(float(val) for val in str(payload).split(",", 2))
             if self._optimistic_color_mode:
-                self._color_mode = ColorMode.XY
-            self._xy_color = xy_color
-            self.async_write_ha_state()
+                self._attr_color_mode = ColorMode.XY
+            self._attr_xy_color = cast(tuple[float, float], xy_color)
+            get_mqtt_data(self.hass).state_write_requests.write_state_request(self)
 
         add_topic(CONF_XY_STATE_TOPIC, xy_received)
 
@@ -621,21 +650,23 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
             self.hass, self._sub_state, topics
         )
 
-    async def _subscribe_topics(self):
+    async def _subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
         await subscription.async_subscribe_topics(self.hass, self._sub_state)
         last_state = await self.async_get_last_state()
 
-        def restore_state(attribute, condition_attribute=None):
+        def restore_state(
+            attribute: str, condition_attribute: str | None = None
+        ) -> None:
             """Restore a state attribute."""
             if condition_attribute is None:
                 condition_attribute = attribute
             optimistic = self._is_optimistic(condition_attribute)
             if optimistic and last_state and last_state.attributes.get(attribute):
-                setattr(self, f"_{attribute}", last_state.attributes[attribute])
+                setattr(self, f"_attr_{attribute}", last_state.attributes[attribute])
 
         if self._topic[CONF_STATE_TOPIC] is None and self._optimistic and last_state:
-            self._state = last_state.state == STATE_ON
+            self._attr_is_on = last_state.state == STATE_ON
         restore_state(ATTR_BRIGHTNESS)
         restore_state(ATTR_RGB_COLOR)
         restore_state(ATTR_HS_COLOR, ATTR_RGB_COLOR)
@@ -649,111 +680,32 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
         restore_state(ATTR_HS_COLOR, ATTR_XY_COLOR)
 
     @property
-    def brightness(self):
-        """Return the brightness of this light between 0..255."""
-        if brightness := self._brightness:
-            brightness = min(round(brightness), 255)
-        return brightness
-
-    @property
-    def color_mode(self):
-        """Return current color mode."""
-        return self._color_mode
-
-    @property
-    def hs_color(self):
-        """Return the hs color value."""
-        return self._hs_color
-
-    @property
-    def rgb_color(self):
-        """Return the rgb color value."""
-        return self._rgb_color
-
-    @property
-    def rgbw_color(self):
-        """Return the rgbw color value."""
-        return self._rgbw_color
-
-    @property
-    def rgbww_color(self):
-        """Return the rgbww color value."""
-        return self._rgbww_color
-
-    @property
-    def xy_color(self):
-        """Return the xy color value."""
-        return self._xy_color
-
-    @property
-    def color_temp(self):
-        """Return the color temperature in mired."""
-        return self._color_temp
-
-    @property
-    def min_mireds(self):
-        """Return the coldest color_temp that this light supports."""
-        return self._config.get(CONF_MIN_MIREDS, super().min_mireds)
-
-    @property
-    def max_mireds(self):
-        """Return the warmest color_temp that this light supports."""
-        return self._config.get(CONF_MAX_MIREDS, super().max_mireds)
-
-    @property
-    def is_on(self):
-        """Return true if device is on."""
-        return self._state
-
-    @property
-    def assumed_state(self):
+    def assumed_state(self) -> bool:
         """Return true if we do optimistic updates."""
         return self._optimistic
 
-    @property
-    def effect_list(self):
-        """Return the list of supported effects."""
-        return self._config.get(CONF_EFFECT_LIST)
-
-    @property
-    def effect(self):
-        """Return the current effect."""
-        return self._effect
-
-    @property
-    def supported_color_modes(self):
-        """Flag supported color modes."""
-        return self._supported_color_modes
-
-    @property
-    def supported_features(self):
-        """Flag supported features."""
-        supported_features = 0
-        supported_features |= (
-            self._topic[CONF_EFFECT_COMMAND_TOPIC] is not None
-            and LightEntityFeature.EFFECT
-        )
-        return supported_features
-
-    async def async_turn_on(self, **kwargs):  # noqa: C901
+    async def async_turn_on(self, **kwargs: Any) -> None:  # noqa: C901
         """Turn the device on.
 
         This method is a coroutine.
         """
         should_update = False
-        on_command_type = self._config[CONF_ON_COMMAND_TYPE]
+        on_command_type: str = self._config[CONF_ON_COMMAND_TYPE]
 
-        async def publish(topic, payload):
+        async def publish(topic: str, payload: PublishPayloadType) -> None:
             """Publish an MQTT message."""
             await self.async_publish(
-                self._topic[topic],
+                str(self._topic[topic]),
                 payload,
                 self._config[CONF_QOS],
                 self._config[CONF_RETAIN],
                 self._config[CONF_ENCODING],
             )
 
-        def scale_rgbx(color, brightness=None):
+        def scale_rgbx(
+            color: tuple[int, ...],
+            brightness: int | None = None,
+        ) -> tuple[int, ...]:
             """Scale RGBx for brightness."""
             if brightness is None:
                 # If there's a brightness topic set, we don't want to scale the RGBx
@@ -761,33 +713,39 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
                 if self._topic[CONF_BRIGHTNESS_COMMAND_TOPIC] is not None:
                     brightness = 255
                 else:
-                    brightness = kwargs.get(
-                        ATTR_BRIGHTNESS, self._brightness if self._brightness else 255
-                    )
+                    brightness = kwargs.get(ATTR_BRIGHTNESS) or self.brightness or 255
             return tuple(int(channel * brightness / 255) for channel in color)
 
-        def render_rgbx(color, template, color_mode):
+        def render_rgbx(
+            color: tuple[int, ...],
+            template: str,
+            color_mode: ColorMode,
+        ) -> PublishPayloadType:
             """Render RGBx payload."""
-            if tpl := self._command_templates[template]:
-                keys = ["red", "green", "blue"]
-                if color_mode == ColorMode.RGBW:
-                    keys.append("white")
-                elif color_mode == ColorMode.RGBWW:
-                    keys.extend(["cold_white", "warm_white"])
-                rgb_color_str = tpl(variables=zip(keys, color))
-            else:
-                rgb_color_str = ",".join(str(channel) for channel in color)
-            return rgb_color_str
+            rgb_color_str = ",".join(str(channel) for channel in color)
+            keys = ["red", "green", "blue"]
+            if color_mode == ColorMode.RGBW:
+                keys.append("white")
+            elif color_mode == ColorMode.RGBWW:
+                keys.extend(["cold_white", "warm_white"])
+            variables = dict(zip(keys, color))
+            return self._command_templates[template](rgb_color_str, variables)
 
-        def set_optimistic(attribute, value, color_mode=None, condition_attribute=None):
+        def set_optimistic(
+            attribute: str,
+            value: Any,
+            color_mode: ColorMode | None = None,
+            condition_attribute: str | None = None,
+        ) -> bool:
             """Optimistically update a state attribute."""
             if condition_attribute is None:
                 condition_attribute = attribute
             if not self._is_optimistic(condition_attribute):
                 return False
             if color_mode and self._optimistic_color_mode:
-                self._color_mode = color_mode
-            setattr(self, f"_{attribute}", value)
+                self._attr_color_mode = color_mode
+
+            setattr(self, f"_attr_{attribute}", value)
             return True
 
         if on_command_type == "first":
@@ -802,14 +760,15 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
             and ATTR_BRIGHTNESS not in kwargs
             and ATTR_WHITE not in kwargs
         ):
-            kwargs[ATTR_BRIGHTNESS] = self._brightness if self._brightness else 255
+            kwargs[ATTR_BRIGHTNESS] = self.brightness or 255
 
-        hs_color = kwargs.get(ATTR_HS_COLOR)
+        hs_color: str | None = kwargs.get(ATTR_HS_COLOR)
 
         if hs_color and self._topic[CONF_HS_COMMAND_TOPIC] is not None:
             await publish(CONF_HS_COMMAND_TOPIC, f"{hs_color[0]},{hs_color[1]}")
             should_update |= set_optimistic(ATTR_HS_COLOR, hs_color, ColorMode.HS)
 
+        rgb: tuple[int, int, int] | None
         if (rgb := kwargs.get(ATTR_RGB_COLOR)) and self._topic[
             CONF_RGB_COMMAND_TOPIC
         ] is not None:
@@ -818,6 +777,7 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
             await publish(CONF_RGB_COMMAND_TOPIC, rgb_s)
             should_update |= set_optimistic(ATTR_RGB_COLOR, rgb, ColorMode.RGB)
 
+        rgbw: tuple[int, int, int, int] | None
         if (rgbw := kwargs.get(ATTR_RGBW_COLOR)) and self._topic[
             CONF_RGBW_COMMAND_TOPIC
         ] is not None:
@@ -826,6 +786,7 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
             await publish(CONF_RGBW_COMMAND_TOPIC, rgbw_s)
             should_update |= set_optimistic(ATTR_RGBW_COLOR, rgbw, ColorMode.RGBW)
 
+        rgbww: tuple[int, int, int, int, int] | None
         if (rgbww := kwargs.get(ATTR_RGBWW_COLOR)) and self._topic[
             CONF_RGBWW_COMMAND_TOPIC
         ] is not None:
@@ -834,6 +795,7 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
             await publish(CONF_RGBWW_COMMAND_TOPIC, rgbww_s)
             should_update |= set_optimistic(ATTR_RGBWW_COLOR, rgbww, ColorMode.RGBWW)
 
+        xy_color: tuple[float, float] | None
         if (xy_color := kwargs.get(ATTR_XY_COLOR)) and self._topic[
             CONF_XY_COMMAND_TOPIC
         ] is not None:
@@ -844,25 +806,25 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
             ATTR_BRIGHTNESS in kwargs
             and self._topic[CONF_BRIGHTNESS_COMMAND_TOPIC] is not None
         ):
-            brightness_normalized = kwargs[ATTR_BRIGHTNESS] / 255
-            brightness_scale = self._config[CONF_BRIGHTNESS_SCALE]
+            brightness_normalized: float = kwargs[ATTR_BRIGHTNESS] / 255
+            brightness_scale: int = self._config[CONF_BRIGHTNESS_SCALE]
             device_brightness = min(
                 round(brightness_normalized * brightness_scale), brightness_scale
             )
             # Make sure the brightness is not rounded down to 0
             device_brightness = max(device_brightness, 1)
-            if tpl := self._command_templates[CONF_BRIGHTNESS_COMMAND_TEMPLATE]:
-                device_brightness = tpl(variables={"value": device_brightness})
-            await publish(CONF_BRIGHTNESS_COMMAND_TOPIC, device_brightness)
+            command_tpl = self._command_templates[CONF_BRIGHTNESS_COMMAND_TEMPLATE]
+            device_brightness_payload = command_tpl(device_brightness, None)
+            await publish(CONF_BRIGHTNESS_COMMAND_TOPIC, device_brightness_payload)
             should_update |= set_optimistic(ATTR_BRIGHTNESS, kwargs[ATTR_BRIGHTNESS])
         elif (
             ATTR_BRIGHTNESS in kwargs
             and ATTR_RGB_COLOR not in kwargs
             and self._topic[CONF_RGB_COMMAND_TOPIC] is not None
         ):
-            rgb_color = self._rgb_color if self._rgb_color is not None else (255,) * 3
-            rgb = scale_rgbx(rgb_color, kwargs[ATTR_BRIGHTNESS])
-            rgb_s = render_rgbx(rgb, CONF_RGB_COMMAND_TEMPLATE, ColorMode.RGB)
+            rgb_color = self.rgb_color or (255,) * 3
+            rgb_scaled = scale_rgbx(rgb_color, kwargs[ATTR_BRIGHTNESS])
+            rgb_s = render_rgbx(rgb_scaled, CONF_RGB_COMMAND_TEMPLATE, ColorMode.RGB)
             await publish(CONF_RGB_COMMAND_TOPIC, rgb_s)
             should_update |= set_optimistic(ATTR_BRIGHTNESS, kwargs[ATTR_BRIGHTNESS])
         elif (
@@ -870,11 +832,9 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
             and ATTR_RGBW_COLOR not in kwargs
             and self._topic[CONF_RGBW_COMMAND_TOPIC] is not None
         ):
-            rgbw_color = (
-                self._rgbw_color if self._rgbw_color is not None else (255,) * 4
-            )
-            rgbw = scale_rgbx(rgbw_color, kwargs[ATTR_BRIGHTNESS])
-            rgbw_s = render_rgbx(rgbw, CONF_RGBW_COMMAND_TEMPLATE, ColorMode.RGBW)
+            rgbw_color = self.rgbw_color or (255,) * 4
+            rgbw_b = scale_rgbx(rgbw_color, kwargs[ATTR_BRIGHTNESS])
+            rgbw_s = render_rgbx(rgbw_b, CONF_RGBW_COMMAND_TEMPLATE, ColorMode.RGBW)
             await publish(CONF_RGBW_COMMAND_TOPIC, rgbw_s)
             should_update |= set_optimistic(ATTR_BRIGHTNESS, kwargs[ATTR_BRIGHTNESS])
         elif (
@@ -882,37 +842,36 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
             and ATTR_RGBWW_COLOR not in kwargs
             and self._topic[CONF_RGBWW_COMMAND_TOPIC] is not None
         ):
-            rgbww_color = (
-                self._rgbww_color if self._rgbww_color is not None else (255,) * 5
-            )
-            rgbww = scale_rgbx(rgbww_color, kwargs[ATTR_BRIGHTNESS])
-            rgbww_s = render_rgbx(rgbww, CONF_RGBWW_COMMAND_TEMPLATE, ColorMode.RGBWW)
+            rgbww_color = self.rgbww_color or (255,) * 5
+            rgbww_b = scale_rgbx(rgbww_color, kwargs[ATTR_BRIGHTNESS])
+            rgbww_s = render_rgbx(rgbww_b, CONF_RGBWW_COMMAND_TEMPLATE, ColorMode.RGBWW)
             await publish(CONF_RGBWW_COMMAND_TOPIC, rgbww_s)
             should_update |= set_optimistic(ATTR_BRIGHTNESS, kwargs[ATTR_BRIGHTNESS])
         if (
             ATTR_COLOR_TEMP in kwargs
             and self._topic[CONF_COLOR_TEMP_COMMAND_TOPIC] is not None
         ):
-            color_temp = int(kwargs[ATTR_COLOR_TEMP])
-            if tpl := self._command_templates[CONF_COLOR_TEMP_COMMAND_TEMPLATE]:
-                color_temp = tpl(variables={"value": color_temp})
-
+            ct_command_tpl = self._command_templates[CONF_COLOR_TEMP_COMMAND_TEMPLATE]
+            color_temp = ct_command_tpl(int(kwargs[ATTR_COLOR_TEMP]), None)
             await publish(CONF_COLOR_TEMP_COMMAND_TOPIC, color_temp)
             should_update |= set_optimistic(
                 ATTR_COLOR_TEMP, kwargs[ATTR_COLOR_TEMP], ColorMode.COLOR_TEMP
             )
 
-        if ATTR_EFFECT in kwargs and self._topic[CONF_EFFECT_COMMAND_TOPIC] is not None:
-            effect = kwargs[ATTR_EFFECT]
-            if effect in self._config.get(CONF_EFFECT_LIST):
-                if tpl := self._command_templates[CONF_EFFECT_COMMAND_TEMPLATE]:
-                    effect = tpl(variables={"value": effect})
+        if (
+            ATTR_EFFECT in kwargs
+            and self._topic[CONF_EFFECT_COMMAND_TOPIC] is not None
+            and CONF_EFFECT_LIST in self._config
+        ):
+            if kwargs[ATTR_EFFECT] in self._config[CONF_EFFECT_LIST]:
+                eff_command_tpl = self._command_templates[CONF_EFFECT_COMMAND_TEMPLATE]
+                effect = eff_command_tpl(kwargs[ATTR_EFFECT], None)
                 await publish(CONF_EFFECT_COMMAND_TOPIC, effect)
                 should_update |= set_optimistic(ATTR_EFFECT, kwargs[ATTR_EFFECT])
 
         if ATTR_WHITE in kwargs and self._topic[CONF_WHITE_COMMAND_TOPIC] is not None:
             percent_white = float(kwargs[ATTR_WHITE]) / 255
-            white_scale = self._config[CONF_WHITE_SCALE]
+            white_scale: int = self._config[CONF_WHITE_SCALE]
             device_white_value = min(round(percent_white * white_scale), white_scale)
             await publish(CONF_WHITE_COMMAND_TOPIC, device_white_value)
             should_update |= set_optimistic(
@@ -927,19 +886,19 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
 
         if self._optimistic:
             # Optimistically assume that the light has changed state.
-            self._state = True
+            self._attr_is_on = True
             should_update = True
 
         if should_update:
             self.async_write_ha_state()
 
-    async def async_turn_off(self, **kwargs):
+    async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn the device off.
 
         This method is a coroutine.
         """
         await self.async_publish(
-            self._topic[CONF_COMMAND_TOPIC],
+            str(self._topic[CONF_COMMAND_TOPIC]),
             self._payload["off"],
             self._config[CONF_QOS],
             self._config[CONF_RETAIN],
@@ -948,5 +907,5 @@ class MqttLight(MqttEntity, LightEntity, RestoreEntity):
 
         if self._optimistic:
             # Optimistically assume that the light has changed state.
-            self._state = False
+            self._attr_is_on = False
             self.async_write_ha_state()
