@@ -96,35 +96,24 @@ from homeassistant.const import (
     ELECTRIC_CURRENT_MILLIAMPERE,
     ELECTRIC_POTENTIAL_MILLIVOLT,
     ELECTRIC_POTENTIAL_VOLT,
-    ENERGY_KILO_WATT_HOUR,
     FREQUENCY_HERTZ,
     FREQUENCY_KILOHERTZ,
     IRRADIATION_WATTS_PER_SQUARE_METER,
-    LENGTH_CENTIMETERS,
-    LENGTH_FEET,
-    LENGTH_METERS,
     LIGHT_LUX,
-    MASS_KILOGRAMS,
-    MASS_POUNDS,
     PERCENTAGE,
-    POWER_BTU_PER_HOUR,
-    POWER_WATT,
-    PRESSURE_INHG,
-    PRESSURE_MMHG,
-    PRESSURE_PSI,
     SIGNAL_STRENGTH_DECIBELS,
     SIGNAL_STRENGTH_DECIBELS_MILLIWATT,
-    SPEED_METERS_PER_SECOND,
-    SPEED_MILES_PER_HOUR,
-    TEMP_CELSIUS,
-    TEMP_FAHRENHEIT,
     TIME_SECONDS,
-    VOLUME_CUBIC_FEET,
-    VOLUME_CUBIC_METERS,
     VOLUME_FLOW_RATE_CUBIC_FEET_PER_MINUTE,
     VOLUME_FLOW_RATE_CUBIC_METERS_PER_HOUR,
-    VOLUME_GALLONS,
-    VOLUME_LITERS,
+    UnitOfEnergy,
+    UnitOfLength,
+    UnitOfMass,
+    UnitOfPower,
+    UnitOfPressure,
+    UnitOfSpeed,
+    UnitOfTemperature,
+    UnitOfVolume,
     UnitOfVolumetricFlux,
 )
 
@@ -173,21 +162,21 @@ MULTILEVEL_SENSOR_DEVICE_CLASS_MAP: dict[str, set[MultilevelSensorType]] = {
 
 METER_UNIT_MAP: dict[str, set[MeterScaleType]] = {
     ELECTRIC_CURRENT_AMPERE: METER_UNIT_AMPERE,
-    VOLUME_CUBIC_FEET: UNIT_CUBIC_FEET,
-    VOLUME_CUBIC_METERS: METER_UNIT_CUBIC_METER,
-    VOLUME_GALLONS: UNIT_US_GALLON,
-    ENERGY_KILO_WATT_HOUR: UNIT_KILOWATT_HOUR,
+    UnitOfVolume.CUBIC_FEET: UNIT_CUBIC_FEET,
+    UnitOfVolume.CUBIC_METERS: METER_UNIT_CUBIC_METER,
+    UnitOfVolume.GALLONS: UNIT_US_GALLON,
+    UnitOfEnergy.KILO_WATT_HOUR: UNIT_KILOWATT_HOUR,
     ELECTRIC_POTENTIAL_VOLT: METER_UNIT_VOLT,
-    POWER_WATT: METER_UNIT_WATT,
+    UnitOfPower.WATT: METER_UNIT_WATT,
 }
 
 MULTILEVEL_SENSOR_UNIT_MAP: dict[str, set[MultilevelSensorScaleType]] = {
     ELECTRIC_CURRENT_AMPERE: SENSOR_UNIT_AMPERE,
-    POWER_BTU_PER_HOUR: UNIT_BTU_H,
-    TEMP_CELSIUS: UNIT_CELSIUS,
-    LENGTH_CENTIMETERS: UNIT_CENTIMETER,
+    UnitOfPower.BTU_PER_HOUR: UNIT_BTU_H,
+    UnitOfTemperature.CELSIUS: UNIT_CELSIUS,
+    UnitOfLength.CENTIMETERS: UNIT_CENTIMETER,
     VOLUME_FLOW_RATE_CUBIC_FEET_PER_MINUTE: UNIT_CUBIC_FEET_PER_MINUTE,
-    VOLUME_CUBIC_METERS: SENSOR_UNIT_CUBIC_METER,
+    UnitOfVolume.CUBIC_METERS: SENSOR_UNIT_CUBIC_METER,
     VOLUME_FLOW_RATE_CUBIC_METERS_PER_HOUR: UNIT_CUBIC_METER_PER_HOUR,
     SIGNAL_STRENGTH_DECIBELS: UNIT_DECIBEL,
     DEGREE: UNIT_DEGREES,
@@ -195,31 +184,31 @@ MULTILEVEL_SENSOR_UNIT_MAP: dict[str, set[MultilevelSensorScaleType]] = {
         *UNIT_DENSITY,
         *UNIT_MICROGRAM_PER_CUBIC_METER,
     },
-    TEMP_FAHRENHEIT: UNIT_FAHRENHEIT,
-    LENGTH_FEET: UNIT_FEET,
-    VOLUME_GALLONS: UNIT_GALLONS,
+    UnitOfTemperature.FAHRENHEIT: UNIT_FAHRENHEIT,
+    UnitOfLength.FEET: UNIT_FEET,
+    UnitOfVolume.GALLONS: UNIT_GALLONS,
     FREQUENCY_HERTZ: UNIT_HERTZ,
-    PRESSURE_INHG: UNIT_INCHES_OF_MERCURY,
+    UnitOfPressure.INHG: UNIT_INCHES_OF_MERCURY,
     UnitOfVolumetricFlux.INCHES_PER_HOUR: UNIT_INCHES_PER_HOUR,
-    MASS_KILOGRAMS: UNIT_KILOGRAM,
+    UnitOfMass.KILOGRAMS: UNIT_KILOGRAM,
     FREQUENCY_KILOHERTZ: UNIT_KILOHERTZ,
-    VOLUME_LITERS: UNIT_LITER,
+    UnitOfVolume.LITERS: UNIT_LITER,
     LIGHT_LUX: UNIT_LUX,
-    LENGTH_METERS: UNIT_METER,
+    UnitOfLength.METERS: UNIT_METER,
     ELECTRIC_CURRENT_MILLIAMPERE: UNIT_MILLIAMPERE,
     UnitOfVolumetricFlux.MILLIMETERS_PER_HOUR: UNIT_MILLIMETER_HOUR,
     ELECTRIC_POTENTIAL_MILLIVOLT: UNIT_MILLIVOLT,
-    SPEED_MILES_PER_HOUR: UNIT_MPH,
-    SPEED_METERS_PER_SECOND: UNIT_M_S,
+    UnitOfSpeed.MILES_PER_HOUR: UNIT_MPH,
+    UnitOfSpeed.METERS_PER_SECOND: UNIT_M_S,
     CONCENTRATION_PARTS_PER_MILLION: UNIT_PARTS_MILLION,
     PERCENTAGE: {*UNIT_PERCENTAGE_VALUE, *UNIT_RSSI},
-    MASS_POUNDS: UNIT_POUNDS,
-    PRESSURE_PSI: UNIT_POUND_PER_SQUARE_INCH,
+    UnitOfMass.POUNDS: UNIT_POUNDS,
+    UnitOfPressure.PSI: UNIT_POUND_PER_SQUARE_INCH,
     SIGNAL_STRENGTH_DECIBELS_MILLIWATT: UNIT_POWER_LEVEL,
     TIME_SECONDS: UNIT_SECOND,
-    PRESSURE_MMHG: UNIT_SYSTOLIC,
+    UnitOfPressure.MMHG: UNIT_SYSTOLIC,
     ELECTRIC_POTENTIAL_VOLT: SENSOR_UNIT_VOLT,
-    POWER_WATT: SENSOR_UNIT_WATT,
+    UnitOfPower.WATT: SENSOR_UNIT_WATT,
     IRRADIATION_WATTS_PER_SQUARE_METER: UNIT_WATT_PER_SQUARE_METER,
 }
 
